#!/bin/sh
# the following restarts using tclsh \
exec tclsh "$0" "$@"

#
# load the library
#

load .libs/libtclgpgme.so


if {[llength $argv] > 0} {
    set pass [lindex $argv 0]
    set argv [lrange $argv 1 end]
}
if {[llength $argv] > 0} {
    set user [lindex $argv 0]
    set argv [lrange $argv 1 end]
} else {
    set user mrose@jabber.dbc.mtview.ca.us
}


package require base64


#
# globals
#

global errorCode

proc callback {label data} {
    global pass

    puts stdout "callback $label:"
    foreach {k v} $data {
        puts stdout [format "    %12.12s: %s" $k $v]
    }
    puts stdout ""

    return $pass
}


proc printkey {ctx key} {
    foreach {k v} [$ctx -operation info-key -key $key] {
        switch -- $k {
            subkeys {
                set i 0
                foreach subkey $v {
                    puts stdout "subkey #[incr i]"

                    foreach {k v} $subkey {
                        puts stdout [format "    %12.12s: %s" $k $v]
                    }
                }
            }

            default {
                puts stdout [format "    %12.12s: %s" $k $v]
            }
        }
    }
}

proc print_info {ctx} {
    if {[string compare [set s [$ctx -operation get -property last-op-info]] \
                ""]} {
        puts stdout ""
        puts stdout "info on last operation"
        puts stdout $s
    }
    puts stdout ""
}


#
# print out available commands and options
#

puts stdout "initializing package...\n"

puts -nonewline stdout [format "%20.20s:" "commands"]
set i 0
foreach p [lsort -dictionary [info commands ::gpgme::*]] {
    puts -nonewline stdout [format "    %-20.20s" \
                                   [string range $p 9 end]]
    if {[expr [incr i]%2] == 0} {
        puts -nonewline stdout [format "\n%20.20s " ""]
    }
}
puts stdout "\n"

foreach p [lsort -dictionary [gpgme::info]]  {
    puts -nonewline stdout [format "%20.20s:" $p]
    set q [lsort -dictionary [gpgme::info $p]]
    set i 0
    while {1} {
        puts -nonewline stdout [format "    %-20.20s" [lindex $q 0]]
        if {[llength [set q [lrange $q 1 end]]] == 0} {
            break
        }
        if {[expr [incr i]%2] == 0} {
            puts -nonewline stdout [format "\n%20.20s " ""]
        }
    }
    puts stdout "\n"
}
puts stdout ""


puts stdout "miscellaneous functions...\n"
foreach {k v} [list check_version        0.3.8                   \
                    engine_check_version [gpgme::info protocols] \
                    get_engine_info      ""] {
    if {[llength $v] == 0} {
        puts -nonewline stdout [format "%20.20s:    " $k]
        if {[catch { gpgme::$k } result]} {
            append result ", $errorCode"
        }
        if {[string compare $result ""]} {
            regsub -all "\n" $result "" result
            puts -nonewline $result
        }
        puts stdout ""
    } else {
        foreach p $v {
            if {[string length [set s "$k $p"]] > 20} {
                puts -nonewline stdout [format "%s:\n%25.25s" "$s" ""]
                set nlP 1
            } else {
                puts -nonewline stdout [format "%20.20s:    " $s]
                set nlP 0
            }
            if {[catch { gpgme::$k $p } result]} {
                append result ", $errorCode"
            }
            if {[string compare $result ""]} {
                puts -nonewline stdout $result
            }
            puts stdout ""
        }
    }
}
puts stdout "\n"


#
# contexts
#

puts stdout "initializing context...\n"
set ctx [gpgme::context]

puts -nonewline stdout [format "%20.20s:" "operations"]
set i 0
foreach p [lsort -dictionary [$ctx -operation info]] {
    puts -nonewline stdout [format "    %-20.20s" $p]
    if {[expr [incr i]%2] == 0} {
        puts -nonewline stdout [format "\n%20.20s " ""]
    }
}
puts stdout ""

foreach p [list get set] {
    puts -nonewline stdout [format "%20.20s:" $p]
    set q [lsort -dictionary [$ctx -operation $p]]
    set i 0
    while {1} {
        puts -nonewline stdout [format "    %-20.20s" [lindex $q 0]]
        if {[llength [set q [lrange $q 1 end]]] == 0} {
            break
        }
        if {[expr [incr i]%2] == 0} {
            puts -nonewline stdout [format "\n%20.20s " ""]
        }
    }
    puts stdout "\n"
}

set keys {}
$ctx -operation start-key -secretonly true -patterns [list $user]
while {[string compare [set key [$ctx -operation next-key]] ""]} {
    lappend keys $key
}
$ctx -operation done-key

$ctx -operation set -property signers -value $keys

foreach p [lsort -dictionary [$ctx -operation get]] {
    puts stdout \
         [format "%20.20s:    %s" $p [$ctx -operation get -property $p]]
}
puts stdout "\n"


#
# keys
#

set i 0
foreach key [$ctx -operation get -property signers] {
    puts stdout "key #[incr i]"
    printkey $ctx $key
    puts stdout "\n"
}


#
# trust items
#

$ctx -operation start-trustitem -pattern test
set i 0
while {[string compare [set item [$ctx -operation next-trustitem]] ""]} {
    puts stdout "trustitem #[incr i]"
    foreach {k v} [$ctx -operation info-trustitem $item] {
        puts stdout [format "    %12.12s: %s" $k $v]
    }
    puts stdout "\n"
}
$ctx -operation done-trustitem


#
# recipients
#

puts stdout "initializing recipients...\n"
set recip [gpgme::recipient]

puts -nonewline stdout [format "%20.20s:" "operations"]
set i 0
foreach p [lsort -dictionary [$recip -operation info]] {
    puts -nonewline stdout [format "    %-20.20s" $p]
    if {[expr [incr i]%2] == 0} {
        puts -nonewline stdout [format "\n%20.20s " ""]
    }
}
puts stdout "\n"

puts stdout "adding $user"
$recip -operation add -name $user -validity full

puts stdout "recipients now has [$recip -operation count] entry"

puts stdout "looping through recipients"
foreach r [$recip -operation list] {
    puts stdout "    $r"
}
puts stdout "done"
puts stdout "\n"


#
# test sign/verify
#

puts stdout "simple tests...\n"

$ctx -operation set -property passphrase-callback -value "callback password"
$ctx -operation set -property progress-callback   -value "callback progress"
$ctx -operation set -property idle-callback       -value "callback idle"


puts stdout "operation: sign"
set input "Hello world.\n"
set signature [$ctx -operation sign -input $input -mode detach]

puts stdout "signature: "
puts stdout [base64::encode $signature]
print_info $ctx


puts stdout "operation: verify"
foreach {k v} [$ctx -operation verify -input $input -signature $signature] {
    switch -- $k {
        signatures {
            foreach signature $v {
                puts stdout "\ninfo on signature"
                foreach {k v} $signature {
                    switch -- $k {
                        key {
                            printkey $ctx $v
                        }

                        default {
                            puts stdout [format "    %12.12s: %s" $k $v]
                        }
                    }
                }
            }
            puts stdout ""
        }

        default {
            puts stdout [format "    %12.12s: %s" $k $v]
        }
    }
}
print_info $ctx


puts stdout "operation: encrypt"
set ciphertext [$ctx -operation encrypt -input $input -recipients $recip]

puts stdout "ciphertext: "
puts stdout [base64::encode $ciphertext]
print_info $ctx


puts stdout "operation: decrypt"
foreach {k v} [$ctx -operation decrypt -input $ciphertext -checkstatus yes] {
    puts stdout [format "    %12.12s: %s" $k $v]
}
print_info $ctx

rename $ctx {}
